;===============================================================================
;
; runic forever 2.1
;
;===============================================================================

header
hirom

;===============================================================================
;
; runic and retort functions
;
;===============================================================================

;-------------------------------------------------------------------------------
; make runic and retort permanent until activation
;-------------------------------------------------------------------------------
org     $C2032E

    AND #$FF                ;make runic and retort not be cleared every turn

warnpc  $C20330

;===============================================================================
;
; runic functions
;
;===============================================================================
org     $C2352B

;-------------------------------------------------------------------------------
;--     runic function
;-------------------------------------------------------------------------------
runic:

    ;check if spell can be absorbed
        LDA $11A3
        BIT #$08            ;runic flag
        BEQ .exit           ;exit if spell can't be absorbed

    ;check all fighters for runic
        LDY #$12            ;fighter ID
    .loop
        JSR check_runic     ;check if fighter can runic
        BCS set_runic       ;setup runic if available
        DEY
        DEY
        BPL .loop           ;try next fighter for runic
        
    ;exit
    .exit
        RTS                 ;exit without available runic

;-------------------------------------------------------------------------------
; check runic for fighter Y
;-------------------------------------------------------------------------------
check_runic:

        LDA $3AA0,Y
        LSR                 ;check for valid target
        BCC .runic_no       ;don't runic if invalid target
        
        PEA $80C0           ;check for sleep, death, petrify
        PEA $2210           ;check for freeze, hide, stop
        JSR $5864           ;routine to check for status
        BCC .runic_no       ;don't runic if any status found
        
        LDA $3E4C,Y
        BIT #$06            ;check for runic flags
        BEQ .runic_no       ;don't runic if spell can't be absorbed

        STX $EE             ;spell target
        TYA                 ;runic user
        CMP $EE             ;compare target with runic user
        BEQ .runic_no       ;don't runic if target is the runic user

	.runic_yes
        SEC
        RTS
        
    .runic_no
        CLC
        RTS

;-------------------------------------------------------------------------------
; setup runic
;-------------------------------------------------------------------------------
set_runic:

        PHX
        JSR $3865
        STZ $3415           ;?randomize target

    ;clear runic flag
        LDA $3E4C,Y
        AND #$FB
        STA $3E4C,Y         ;clear runic flag
        
    ;set runic target
        REP #$20            ;2AB mode
        LDA $3018,Y
        STA $B8             ;save fighter as spell target
        
    ;null spell status
        JSR $520E           ;X = number of targets
        STZ $11AA           ;null status bytes 0,1
        STZ $11AC           ;null status bytes 2,3

    ;set spell proprieties
        LDA #$6000          ;set ignore defense, no split damage, no elemental atributes
        STA $11A1           ;setup it
        LDA #$2182          ;set concern MP, not reflectable, unblockable, heal
        STA $11A3           ;setup it
        
    ;set spell power as mp cost
        SEP #$20            ;1AB mode
        TDC                 ;clear AB
        LDA $11A5           ;MP cost of spell
        JSR $4792           ;divide by X
        STA $11A6           ;save as spell power

    ;miscellaneous adjustments
        JSR $385E           ;sets level and magic power to 0
        JSL runic_misc      ;miscellaneous settings

    ;exit
        PLX
        RTS

warnpc  $C235A9

;===============================================================================
;
; retort functions
;
;===============================================================================

;-------------------------------------------------------------------------------
; setup new retort identifier
;-------------------------------------------------------------------------------
org     $C24CCE

    LDA #$5D

warnpc  $C24CD0

;-------------------------------------------------------------------------------
; swordtech function
;-------------------------------------------------------------------------------
org     $C21857

    ;retort special cases
        LDA $B6         ;swordtech selection
        CMP #$01        ;retort - passive
        BEQ .retort_p
        CMP #$08        ;retort - active
        BEQ .retort_a

    ;execute normal swordtech
.message
        LDA #$04        ;swordtech names
        STA $3412       ;setup it
.execute
        JMP $317B       ;execute move

    ;passive retort
.retort_p:
        LDA $3E4C,X     ;flag
        ORA #$01        ;setup retort flag
        STA $3E4C,X     ;save it
        JSL retort_misc_p
        BRA .execute

    ;active retort
.retort_a:
        LDA #$56        ;retort spell index
        JSR $2966       ;load data spell
        JSL retort_misc_a
        BRA .message

warnpc  $C21885

;===============================================================================
;
; auxiliary functions
;
;===============================================================================
org     $C3FCC0             ;(*) adjust for start of free space

;-------------------------------------------------------------------------------
; runic miscellaneous adjustments
;-------------------------------------------------------------------------------
runic_misc:

        STZ $3414           ;skip damage modification
        LDA #$40
        TRB $B2             ;setup runic animation?
        LDA #$04
        STA $BA             ;don't retarget if target is invalid
        LDA #$03
        TRB $11A7           ;turn off text and miss if status isn't set or clear
        STZ $11A9           ;nulls spell special effect
        RTL

;-------------------------------------------------------------------------------
; passive retort miscellaneous adjustments
;-------------------------------------------------------------------------------
retort_misc_p:

        LDA #$80        ;null action index
        STA $B6         ;setup it
        STZ $11A6       ;null power
        LDA #$20        ;can't evade flag
        TSB $11A4       ;setup it
        LDA #$01        ;magical damage
        TRB $11A2       ;setup it
        RTL

;-------------------------------------------------------------------------------
; active retort miscellaneous adjustments
;-------------------------------------------------------------------------------
retort_misc_a:

    ;retort flag
        LDA $3E4C,X     ;flag
        AND #$FE        ;unset retort flag
        STA $3E4C,X     ;save it

    ;retort extra setup
        LDA #$01        ;retort action
        STA $B6         ;save as swordtech selection
        LDA #$10
        TRB $B0
        RTL

warnpc  $C3FCF7

